--------------------------------------------------------------------------------------------------/
-- Company: Microsemi SoC Products Group
--
-- File: count24.vhd
-- File history:
--      1.0: July 22, 2016: Written
--      2.0: Nov. 28, 2018: UP_DN control signal added
--      3.0: Feb. 10, 2021: DATA_WIDTH generic added
--      <Revision number>: <Date>: <Comments>
--
-- Description: 
--
-- 24 bit counter
--
-- Targeted device: <Family::SmartFusion2> <Die::M2S050T_ES> <Package::896 FBGA>
-- Authors: Luca Cattaneo
--
--------------------------------------------------------------------------------------------------/ 
library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity count24 is
 generic(DATA_WIDTH: integer := 24);
    port(CLK: in std_logic;
         UP_DN: in std_logic;
         RESETn: in std_logic;
         Q: out  std_logic_vector(DATA_WIDTH-1 downto 0));
end count24;

architecture RTL of count24 is

-- signal declarations
signal Q_int: std_logic_vector(DATA_WIDTH-1 downto 0);
signal Q_endcnt: std_logic;
-- add Synplify directive syn_keep and attribute alspreserve to prevent Q[DATA_WIDTH-1:19] from being
-- removed in synthesis or layout.  Without these attributes the counter is pruned because the
-- counter output bits [DATA_WIDTH-1:19] don't drive any logic.  These will be driven off chip using Live Probe.
-- Explicit type declaration
attribute syn_keep : boolean;
attribute syn_keep of Q_int : signal is true;
attribute syn_keep of Q_endcnt : signal is true;
attribute alspreserve : boolean;
attribute alspreserve of Q_int : signal is true;
attribute alspreserve of Q_endcnt : signal is true;

begin
-- free running binary counter
CNT: process (CLK, RESETn)
begin
  if (RESETn = '0') then 
     Q_int <= (others => '0');
     Q_endcnt <= '0';
  elsif rising_edge(CLK) then
     if (UP_DN = '1') then
        Q_int <= Q_int + "01"; -- counting up
     elsif (UP_DN = '0') then
        Q_int <= Q_int - "01"; -- counting down
     else
        Q_int <= Q_int;
     end if;
     if (Q_int(DATA_WIDTH-1 downto 0) = conv_std_logic_vector((2**DATA_WIDTH)-1, DATA_WIDTH)) then
        Q_endcnt <= '1';
     else
        Q_endcnt <= '0';
     end if;   
  end if;
end process CNT;

Q <= Q_int;

end RTL;

